==> hibernate-search-backend-lucene-6.2.4.Final-sources.jar: org/hibernate/search/backend/lucene/types/codec/impl/LuceneStringFieldCodec.java <==
/*
 * Hibernate Search, full-text search for your domain model
 *
 * License: GNU Lesser General Public License (LGPL), version 2.1 or later
 * See the lgpl.txt file in the root directory or <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
package org.hibernate.search.backend.lucene.types.codec.impl;

import org.hibernate.search.backend.lucene.lowlevel.common.impl.AnalyzerConstants;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.FieldType;
import org.apache.lucene.document.SortedSetDocValuesField;
import org.apache.lucene.index.IndexableField;
import org.apache.lucene.util.BytesRef;

public final class LuceneStringFieldCodec implements LuceneStandardFieldCodec<String, String> {

	private final FieldType mainFieldType;
	private final DocValues docValues;
	private final String indexNullAsValue;
	private final Analyzer analyzerOrNormalizer;

	public LuceneStringFieldCodec(FieldType mainFieldType, DocValues docValues,
			String indexNullAsValue, Analyzer analyzerOrNormalizer) {
		this.mainFieldType = mainFieldType;
		this.docValues = docValues;
		this.indexNullAsValue = indexNullAsValue;
		this.analyzerOrNormalizer = analyzerOrNormalizer;
	}

	@Override
	public void addToDocument(LuceneDocumentContent documentBuilder, String absoluteFieldPath, String value) {
		if ( value == null && indexNullAsValue != null ) {
			value = indexNullAsValue;
		}

		if ( value == null ) {
			return;
		}

		if ( mainFieldType != null ) {
			documentBuilder.addField( new Field( absoluteFieldPath, value, mainFieldType ) );
		}

		if ( DocValues.ENABLED.equals( docValues ) ) {
			BytesRef normalized = normalize( absoluteFieldPath, value );
			documentBuilder.addField( new SortedSetDocValuesField( absoluteFieldPath, normalized ) );
		}

		if ( ( mainFieldType == null || mainFieldType.omitNorms() ) && DocValues.DISABLED.equals( docValues ) ) {
			// For the "exists" predicate
			documentBuilder.addFieldName( absoluteFieldPath );
		}
	}

	@Override
	public String decode(IndexableField field) {
		return field.stringValue();
	}

	@Override
	public boolean isCompatibleWith(LuceneFieldCodec<?> obj) {
		if ( this == obj ) {
			return true;
		}
		return LuceneStringFieldCodec.class == obj.getClass();
	}

	@Override
	public String encode(String value) {
		return value;
	}

	private BytesRef normalize(String absoluteFieldPath, String value) {
		if ( value == null ) {
			return null;
		}
		if ( analyzerOrNormalizer == AnalyzerConstants.KEYWORD_ANALYZER ) {
			// Optimization when analysis is disabled
			return new BytesRef( value );
		}
		return analyzerOrNormalizer.normalize( absoluteFieldPath, value );
	}
}

==> hibernate-search-backend-lucene-7.2.4.Final-sources.jar: org/hibernate/search/backend/lucene/types/codec/impl/LuceneStringFieldCodec.java <==
/*
 * SPDX-License-Identifier: Apache-2.0
 * Copyright Red Hat Inc. and Hibernate Authors
 */
package org.hibernate.search.backend.lucene.types.codec.impl;

import org.hibernate.search.backend.lucene.lowlevel.common.impl.AnalyzerConstants;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.FieldType;
import org.apache.lucene.document.SortedSetDocValuesField;
import org.apache.lucene.index.IndexableField;
import org.apache.lucene.util.BytesRef;

public final class LuceneStringFieldCodec implements LuceneFieldCodec<String, String> {

	private final FieldType mainFieldType;
	private final DocValues docValues;
	private final String indexNullAsValue;
	private final Analyzer analyzerOrNormalizer;

	public LuceneStringFieldCodec(FieldType mainFieldType, DocValues docValues,
			String indexNullAsValue, Analyzer analyzerOrNormalizer) {
		this.mainFieldType = mainFieldType;
		this.docValues = docValues;
		this.indexNullAsValue = indexNullAsValue;
		this.analyzerOrNormalizer = analyzerOrNormalizer;
	}

	@Override
	public void addToDocument(LuceneDocumentContent documentBuilder, String absoluteFieldPath, String value) {
		if ( value == null && indexNullAsValue != null ) {
			value = indexNullAsValue;
		}

		if ( value == null ) {
			return;
		}

		if ( mainFieldType != null ) {
			documentBuilder.addField( new Field( absoluteFieldPath, value, mainFieldType ) );
		}

		if ( DocValues.ENABLED.equals( docValues ) ) {
			BytesRef normalized = normalize( absoluteFieldPath, value );
			documentBuilder.addField( new SortedSetDocValuesField( absoluteFieldPath, normalized ) );
		}

		if ( ( mainFieldType == null || mainFieldType.omitNorms() ) && DocValues.DISABLED.equals( docValues ) ) {
			// For the "exists" predicate
			documentBuilder.addFieldName( absoluteFieldPath );
		}
	}

	@Override
	public String decode(IndexableField field) {
		return field.stringValue();
	}

	@Override
	public String raw(IndexableField field) {
		return decode( field );
	}

	@Override
	public String decode(String field) {
		return field;
	}

	@Override
	public boolean isCompatibleWith(LuceneFieldCodec<?, ?> obj) {
		if ( this == obj ) {
			return true;
		}
		return LuceneStringFieldCodec.class == obj.getClass();
	}

	@Override
	public String encode(String value) {
		return value;
	}

	private BytesRef normalize(String absoluteFieldPath, String value) {
		if ( value == null ) {
			return null;
		}
		if ( analyzerOrNormalizer == AnalyzerConstants.KEYWORD_ANALYZER ) {
			// Optimization when analysis is disabled
			return new BytesRef( value );
		}
		return analyzerOrNormalizer.normalize( absoluteFieldPath, value );
	}

	public Class<String> encodedType() {
		return String.class;
	}
}
